@@ -1,14 +1,14 @@
 import tensorflow as tf
 
 
-def keras_model_to_tflitel(model, config):
+def keras_model_to_tflite(model, config):
     """
 
     :param model:
     :param config: (dict) configuration info Ex)
     {
       "quantization": false,
-      "quantization_type": "int8",
+      "quantization_type": "int8",  # ["int8", "float16", "float32"]
       "tf_ops": false,
       "exp_converter": false,
       "out_path": "/writing/tflite/model/path"
@@ -17,6 +17,8 @@ def keras_model_to_tflitel(model, config):
     """
     converter = tf.lite.TFLiteConverter.from_keras_model(model)
 
+    config = parse_config(config)
+
     optimizations = []
     supported_ops = [tf.lite.OpsSet.TFLITE_BUILTINS]
     supported_types = []
@@ -48,3 +50,10 @@ def keras_model_to_tflitel(model, config):
 
     return tflite_model
 
+
+def parse_config(config):
+
+    qtype = config['quantization_type']
+    config['quantization_type'] = tf.float16 if qtype == "float16" else tf.int8 if qtype == "int8" else tf.float32
+
+    return config
